/*
 * x l S Q L  
 * (c) Jim Caprioli, NiLOSTEP.com
 * See xlSQL-license.txt for license details
 *
 */
package com.nilostep.xlsql.jdbc;

import com.nilostep.xlsql.database.sql.ICommand;
import java.sql.*;


public class xlStatement implements Statement {
    //~ Constructors 
    
    protected xlConnection xlCon;
    private Statement dbStm;
    /**
    * Constructs a new StatementImpl object.
    *
    */
    protected xlStatement(xlConnection con, Statement stm) throws SQLException {
         xlCon = con;
         dbStm = stm;
    }

    //~ Methods 

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#addBatch
    */
    public void addBatch(String sql) throws SQLException {
        dbStm.addBatch(sql);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#cancel
    */
    public void cancel() throws SQLException {
        dbStm.cancel();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#clearBatch
    */
    public void clearBatch() throws SQLException {
        dbStm.clearBatch();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#clearWarnings
    */
    public void clearWarnings() throws SQLException {
        dbStm.clearWarnings();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#close
    */
    public void close() throws SQLException {
        dbStm.close();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#execute
    */
    public boolean execute(String sql) throws SQLException {
        boolean ret = true;
        String[] sqlCommand = sql.split("[;]");
        for (int i=0; i < sqlCommand.length; i++) {
            ICommand cmd = xlCon.xlsql.parseSql(sqlCommand[i]);
            if (cmd.execAllowed()) {
                // dbStm may throw an SQLException..., pass on to client
                ret = dbStm.execute(sqlCommand[i]);
                cmd.execute();
            }
            else {
                throw new SQLException("xlSQL: execute not allowed");
            }
        }
        return ret;
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#execute
    */
    public boolean execute(String sql, String[] columnNames)
                    throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#execute
    */
    public boolean execute(String sql, int autoGeneratedKeys)
                    throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#execute
    */
    public boolean execute(String sql, int[] columnIndexes)
                    throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeBatch
    */
    public int[] executeBatch() throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeQuery
    */
    public ResultSet executeQuery(String sql) throws SQLException {
        ResultSet dbRs = dbStm.executeQuery(sql);
        ResultSet rs = new xlResultSet(this, dbRs);
        return rs;
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeUpdate
    */
    public int executeUpdate(String sql) throws SQLException {
        int ret = 0;
        String[] sqlCommand = sql.split("[;]");
        for (int i=0; i < sqlCommand.length; i++) {
            ICommand cmd = xlCon.xlsql.parseSql(sqlCommand[i]);            
            if (cmd.execAllowed()) {
                // dbStm may throw an SQLException..., pass on to client
                ret = dbStm.executeUpdate(sqlCommand[i]);
                cmd.execute();
            }
            else {
                throw new SQLException("xlSQL: execute not allowed");
            }
        }
        return ret;
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeUpdate
    */
    public int executeUpdate(String sql, String[] columnNames)
                      throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeUpdate
    */
    public int executeUpdate(String sql, int autoGeneratedKeys)
                      throws SQLException {
        throw new SQLException("not supported");
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#executeUpdate
    */
    public int executeUpdate(String sql, int[] columnIndexes)
                      throws SQLException {
        throw new SQLException("not supported");
    }
    
    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getConnection
    */
    public Connection getConnection() throws SQLException {
        return xlCon;
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getFetchDirection
    */
    public int getFetchDirection() throws SQLException {
        return dbStm.getFetchDirection();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getFetchSize
    */
    public int getFetchSize() throws SQLException {
        return dbStm.getFetchSize();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getGeneratedKeys
    */
    public ResultSet getGeneratedKeys() throws SQLException {
        return dbStm.getGeneratedKeys();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getMaxFieldSize
    */
    public int getMaxFieldSize() throws SQLException {
        return dbStm.getMaxFieldSize();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getMaxRows
    */
    public int getMaxRows() throws SQLException {
        return dbStm.getMaxRows();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getMoreResults
    */
    public boolean getMoreResults() throws SQLException {
        return dbStm.getMoreResults();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getMoreResults
    */
    public boolean getMoreResults(int current) throws SQLException {
        return dbStm.getMoreResults(current);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getQueryTimeout
    */
    public int getQueryTimeout() throws SQLException {
        return dbStm.getQueryTimeout();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getResultSet
    */
    public ResultSet getResultSet() throws SQLException {
        ResultSet dbRs = dbStm.getResultSet();
        ResultSet rs = new xlResultSet(this, dbRs);
        return rs;
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getResultSetConcurrency
    */
    public int getResultSetConcurrency() throws SQLException {
        return dbStm.getResultSetConcurrency();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getResultSetHoldability
    */
     public int getResultSetHoldability() throws SQLException {
        return dbStm.getResultSetHoldability();
    }
   
    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getResultSetType
    */
    public int getResultSetType() throws SQLException {
        return dbStm.getResultSetType();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getUpdateCount
    */
    public int getUpdateCount() throws SQLException {
        return dbStm.getUpdateCount();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#getWarnings
    */
    public SQLWarning getWarnings() throws SQLException {
        return dbStm.getWarnings();
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setCursorName
    */
    public void setCursorName(String name) throws SQLException {
        dbStm.setCursorName(name);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setEscapeProcessing
    */
    public void setEscapeProcessing(boolean enable) throws SQLException {
        dbStm.setEscapeProcessing(enable);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setFetchDirection
    */
    public void setFetchDirection(int direction) throws SQLException {
        dbStm.setFetchDirection(direction);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setFetchSize
    */
    public void setFetchSize(int rows) throws SQLException {
        dbStm.setFetchSize(rows);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setMaxFieldSize
    */
    public void setMaxFieldSize(int max) throws SQLException {
        dbStm.setMaxFieldSize(max);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setMaxRows
    */
    public void setMaxRows(int max) throws SQLException {
        dbStm.setMaxRows(max);
    }

    /**
    * Implements method in interface java.sql.Statement
    * @see java.sql.Statement#setQueryTimeout
    */
    public void setQueryTimeout(int seconds) throws SQLException {
        dbStm.setQueryTimeout(seconds);
    }

}